@@ -22,26 +22,26 @@ ci <- function(...) {
 }
 
 ci.formula <- function(formula, data, ...) {
-	# Get the data. Use standard code from survival::coxph as suggested by Terry Therneau
-	Call <- match.call()
-	indx <- match(c("formula", "data", "weights", "subset", "na.action"), names(Call), nomatch=0)
-	if (indx[1] == 0) {
-		stop("A formula argument is required")
+	data.missing <- missing(data)
+	roc.data <- roc.utils.extract.formula(formula, data, ..., 
+										  data.missing = data.missing,
+										  call = match.call())
+	if (length(roc.data$predictor.name) > 1) {
+		stop("Only one predictor supported in 'ci'.")
 	}
-	# Keep the standard arguments and run them in model.frame
-	temp <- Call[c(1,indx)]  
-	temp[[1]] <- as.name('model.frame')
-	m <- eval(temp, parent.frame())
-	
-	if (!is.null(model.weights(m))) stop("weights are not supported")
-	
-	response <- model.response(m)
-	predictor <- m[[attr(terms(formula), "term.labels")]]
+	response <- roc.data$response
+	predictor <- roc.data$predictors[, 1]
 	ci.roc(roc(response, predictor, ...), ...)
 }
 
 ci.default <- function(response, predictor, ...) {
-  ci.roc(roc.default(response, predictor, ...), ...)
+	roc <- roc.default(response, predictor, ci = FALSE, ...)
+	if (methods::is(roc, "smooth.roc")) {
+		return(ci.roc(smooth.roc = roc, ...))
+	}
+	else {
+		return(ci.roc(roc = roc, ...))
+	}
 }
 
 ci.smooth.roc <- function(smooth.roc, of = c("auc", "sp", "se", "coords"), ...) {
