@@ -22,39 +22,17 @@ multiclass.roc <- function(...)
 
 multiclass.roc.formula <- function(formula, data, ...) {
 	data.missing <- missing(data)
-	if (data.missing) {
-		predictors <- attr(terms(formula), "term.labels")
+	call <- match.call()
+	roc.data <- roc.utils.extract.formula(formula, data, ..., 
+										  data.missing = data.missing,
+										  call = call)
+	response <- roc.data$response
+	predictors <- roc.data$predictors
+	if (ncol(predictors) == 1) {
+		predictors <- predictors[, 1]
 	}
-	else {
-		predictors <- attr(terms(formula, data = data), "term.labels")
-	}
-	
-	# Get the data. Use standard code from survival::coxph as suggested by Terry Therneau
-	Call <- match.call()
-	indx <- match(c("formula", "data", "weights", "subset", "na.action"), names(Call), nomatch=0)
-	if (indx[1] == 0) {
-		stop("A formula argument is required")
-	}
-	# Keep the standard arguments and run them in model.frame
-	temp <- Call[c(1,indx)]  
-	temp[[1]] <- as.name('model.frame')
-	m <- eval(temp, parent.frame())
-	
-	if (!is.null(model.weights(m))) stop("weights are not supported")
-	
-	# Get response (easy)
-	response <- model.response(m)
-	
-	if (length(response) == 0) {
-		stop("Error in the formula: a response is required in a formula of type response~predictor.")
-	}
-	
-	predictor <- m[predictors]
-	if (ncol(predictor) == 1) {
-		predictor <- as.vector(unlist(predictor))
-	}
-	multiclass.roc <- multiclass.roc.default(response, predictor, ...)
-	multiclass.roc$call <- Call
+	multiclass.roc <- multiclass.roc.default(response, predictors, ...)
+	multiclass.roc$call <- call
 	if (! data.missing) {
 		multiclass.roc$data <- data
 	}
