@@ -358,6 +358,20 @@ test_that("roc reject requires cases & controls", {
 	expect_error(roc(aSAH[aSAH$outcome == "Poor",], outcome, ndka), "control")
 })
 
+
+test_that("roc works with ordered predictor", {
+	wfns2 <- ordered(as.numeric(aSAH$wfns) + 2)
+	r <- roc(aSAH$outcome, wfns2)
+	expect_equal(r$thresholds, c(-Inf, 3.5, 4.5, 5.5, 6.5, Inf))
+	
+	levels(wfns2) <- letters[1:5]
+	# TODO: this behavior should be fixed, see issue #63.
+	# For now ensure basic behavior with warning is at least consistent.
+	expect_warning(r <- roc(aSAH$outcome, wfns2))
+	expect_equal(r$thresholds, c(-Inf, 1.5, 2.5, 3.5, 4.5, Inf))
+	# In reality we want to say something like  c(-Inf, "a", "b", "c", "d", "e", Inf)
+})
+
 # The code below can be used to refresh the "expected.roc" data, just in case...
 # expected.roc <- list()
 # for (marker in c("ndka", "wfns", "s100b")) {
